#######################################################################
#
# NCM component for ldconf daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.4 : 07/12/04 12:36
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::ldconf;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;


local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/ldconf";

    # File to modify (default is /etc/ld.so.conf). 
    my $fname = $config->getValue("$base/conffile");

    # Hash containing all paths (used to make paths unique).
    my %toAdd;

    # If the list of paths exists, get the full list. 
    if ($config->elementExists("$base/paths")) {

        # Get the list of VOs to configure, should be a list.
        my @paths = $config->getElement("$base/paths")->getList();
        foreach my $element (@paths) {
            my $path = $element->getValue();

            # Ensure that value is OK.
            if ($path =~ m/^\s*(\S+)\s*$/) {
                $path = $1;
                $toAdd{$path} = 'new';
            } else {
                $self->warn("bad path ($path) in configuration");
            }
        }
    }

    # Get the existing paths.
    if (-e $fname) {
        my $contents; 
        open TMP, "<$fname";
        while (<TMP>) {
            $contents .= $_;
        }
        close TMP;

        # Split on valid delimiters.
        my @existing = split /[,:_\s\t\n]+/m, $contents;

        # Add these to list.
        foreach (@existing) {
            $toAdd{$_} = 'existing';
        }
    }

    # Determine if there are any changes to be made.
    my $changes = 0;
    foreach my $key (keys %toAdd) {
	$changes = 1 if ($toAdd{$key} eq 'new');
    }

    if ($changes) {

	# Create the merged output.
	my $contents = join("\n",(sort keys %toAdd)) . "\n";
	
	# Now just create the new configuration file.  Be careful to save
	# a backup of the previous file if necessary. 
	if ( ! -e $fname ) {
	    
	    # Configuration file doesn't exist yet.  Create it. 
	    open ( CONF,">$fname" );
	    print CONF $contents;
	    close (CONF);
	    $self->log("$fname created");
	    
	} else {
	    
	    # Already exists. Make backup and create new file. 
	    my $result = LC::Check::file( $fname,
					  backup => ".old",
					  contents => $contents,
					  );
	    $self->log("$fname updated") if $result;
	}
	
	# Make sure to reinitialize the ld.so.conf cache.
	`/sbin/ldconfig`;
	$self->warn("error running ldconfig") if $?;

    } else {
	$self->log("no changes need to be made");
    }
    
    return 1;
}

1;      # Required for PERL modules
